@@ -1,4 +1,3 @@
-import json
 import logging
 import math
 import os
@@ -9,6 +8,7 @@ from datetime import timedelta
 
 import feedparser
 from dateutil import parser
+from six.moves.urllib.parse import unquote
 
 from .ProjectHolder import ProjectHolder
 from .utils import ApiCredentialsError, BadProjectError
@@ -19,10 +19,6 @@ TOKEN_PRO_TIP = 'ProTip: set GITHUB_API_TOKEN env var as per ' \
                 'https://github.com/dvershinin/lastversion#tips'
 
 
-def set_matching_from_tag(ret, tag):
-    ret = tag
-
-
 def asset_matches(asset, search, regex_matching):
     """Check if the asset equals to string or satisfies a regular expression
     Args:
@@ -47,6 +43,12 @@ class GitHubRepoSession(ProjectHolder):
 
     DEFAULT_HOSTNAME = 'github.com'
 
+    TOKEN_ENV_VARS = [
+        "LASTVERSION_GITHUB_API_TOKEN",
+        "GITHUB_API_TOKEN",
+        "GITHUB_TOKEN"
+    ]
+
     # one-word aliases or simply known popular repos to skip using search API
     KNOWN_REPOS_BY_NAME = {
         'php': {
@@ -87,15 +89,11 @@ class GitHubRepoSession(ProjectHolder):
     def find_repo_by_name_only(self, repo):
         if repo.startswith(('https://', 'http://')):
             return None
-        cache_repo_names_file = "{}/repos.json".format(self.cache_dir)
-        try:
-            with open(cache_repo_names_file, 'r') as reader:
-                cache = json.load(reader)
-        except (IOError, ValueError):
-            cache = {}
+        cache = self.get_name_cache()
+
         try:
             if repo in cache and time.time() - cache[repo]['updated_at'] < 3600 * 24 * 30:
-                log.info("Found {} in repo short name cache".format(repo))
+                log.info("Found %s in repo short name cache", repo)
                 if not cache[repo]['repo']:
                     raise BadProjectError(
                         'No project found on GitHub for search query: {}'.format(repo)
@@ -103,7 +101,8 @@ class GitHubRepoSession(ProjectHolder):
                 return cache[repo]['repo']
         except TypeError:
             pass
-        log.info("Making query against GitHub API to search repo {}".format(repo))
+
+        log.info("Making query against GitHub API to search repo %s", repo)
         r = self.get(
             '{}/search/repositories'.format(self.api_base),
             params={
@@ -129,13 +128,9 @@ class GitHubRepoSession(ProjectHolder):
             'repo': full_name,
             'updated_at': int(time.time())
         }
-        try:
-            with open(cache_repo_names_file, 'w') as writer:
-                writer.write(
-                    json.dumps(cache)
-                )
-        except (IOError, ValueError):
-            pass
+
+        self.update_name_cache(cache)
+
         if not full_name:
             raise BadProjectError(
                 'No project found on GitHub for search query: {}'.format(repo)
@@ -149,11 +144,16 @@ class GitHubRepoSession(ProjectHolder):
         # lazy loaded dict cache of /releases response keyed by tag, only first page
         self.formal_releases_by_tag = None
         self.rate_limited_count = 0
-        self.api_token = os.getenv("LASTVERSION_GITHUB_API_TOKEN")
+        self.api_token = None
+        for var_name in self.TOKEN_ENV_VARS:
+            token = os.getenv(var_name)
+            if token:
+                self.api_token = token
+                log.info('Using API token %s.', var_name)
+                self.headers.update({'Authorization': "token {}".format(self.api_token)})
+                break
         if not self.api_token:
-            self.api_token = os.getenv("GITHUB_API_TOKEN")
-        if not self.api_token:
-            self.api_token = os.getenv("GITHUB_TOKEN")
+            log.info('No API token found in environment variables %s.', self.TOKEN_ENV_VARS)
         self.hostname = hostname
         if not self.hostname:
             self.hostname = self.DEFAULT_HOSTNAME
@@ -161,9 +161,7 @@ class GitHubRepoSession(ProjectHolder):
         self.headers.update({
             'Accept': 'application/vnd.github+json'
         })
-        if self.api_token:
-            log.info('Using API token.')
-            self.headers.update({'Authorization': "token {}".format(self.api_token)})
+
         if self.hostname != self.DEFAULT_HOSTNAME:
             self.api_base = 'https://{}/api/v3'.format(self.hostname)
         else:
@@ -172,11 +170,11 @@ class GitHubRepoSession(ProjectHolder):
             official_repo = self.try_get_official(repo)
             if official_repo:
                 repo = official_repo
-                log.info('Using official repo {}'.format(repo))
+                log.info('Using official repo %s', repo)
             else:
                 repo = self.find_repo_by_name_only(repo)
                 if repo:
-                    log.info('Using repo {} obtained from search API'.format(repo))
+                    log.info('Using repo %s obtained from search API', repo)
                 else:
                     return
         self.set_repo(repo)
@@ -187,7 +185,7 @@ class GitHubRepoSession(ProjectHolder):
     def get(self, url, **kwargs):
         """Send GET request and account for GitHub rate limits and such."""
         r = super(GitHubRepoSession, self).get(url, **kwargs)
-        log.info('Got HTTP status code {} from {}'.format(r.status_code, url))
+        log.info('Got HTTP status code %s from %s', r.status_code, url)
         if r.status_code == 401:
             if self.api_token:
                 raise ApiCredentialsError('API request was denied despite using an API token. '
@@ -215,16 +213,15 @@ class GitHubRepoSession(ProjectHolder):
                         )
                     else:
                         w = 'Waiting {} seconds for API quota reinstatement.'.format(wait_for)
-                        if "GITHUB_API_TOKEN" not in os.environ \
-                                and 'GITHUB_TOKEN' not in os.environ:
+                        if not self.api_token:
                             w = "{} {}".format(w, TOKEN_PRO_TIP)
                         log.warning(w)
                         time.sleep(wait_for)
                     self.rate_limited_count = self.rate_limited_count + 1
                     return self.get(url)
                 raise ApiCredentialsError(
-                    'Exceeded API rate limit after waiting: {}'.format(
-                        r.json()['message'])
+                    'Exceeded GitHub API rate limits. Giving up due to high expected wait {}s. API says: {}'.format(
+                        wait_for, r.json()['message'])
                 )
             return self.get(url)
 
@@ -333,11 +330,11 @@ class GitHubRepoSession(ProjectHolder):
             # testing on php/php-src
             owner, name = self.repo.split('/')
             query = query_fmt % (owner, name, cursor)
-            log.info('Running query {}'.format(query))
+            log.info('Running query %s', query)
             r = self.post('{}/graphql'.format(self.api_base), json={'query': query})
-            log.info('Requested graphql with cursor "{}"'.format(cursor))
+            log.info('Requested graphql with cursor "%s"', cursor)
             if r.status_code != 200:
-                log.info("query returned non 200 response code {}".format(r.status_code))
+                log.info("query returned non 200 response code %s", r.status_code)
                 return ret
             j = r.json()
             if 'errors' in j and j['errors'][0]['type'] == 'NOT_FOUND':
@@ -345,7 +342,7 @@ class GitHubRepoSession(ProjectHolder):
                     'No such project found on GitHub: {}'.format(self.repo)
                 )
             if not j['data']['repository']['tags']['edges']:
-                log.info('No tags in GraphQL response: {}'.format(r.text))
+                log.info('No tags in GraphQL response: %s', r.text)
                 break
             for edge in j['data']['repository']['tags']['edges']:
                 node = edge['node']
@@ -364,7 +361,7 @@ class GitHubRepoSession(ProjectHolder):
                     d = node['target']['author']['date']
                 tag_date = parser.parse(d)
                 if ret and tag_date + timedelta(days=365) < ret['tag_date']:
-                    log.info('The version {} is newer, but is too old!'.format(version))
+                    log.info('The version %s is newer, but is too old!', version)
                     break
                 if not ret or version > ret['version'] or tag_date > ret['tag_date']:
                     # we always want to return formal release if it exists, because it has useful
@@ -482,8 +479,7 @@ class GitHubRepoSession(ProjectHolder):
             if r.status_code == 200:
                 repo_data = r.json()
                 if self.repo != repo_data['full_name']:
-                    log.info('Detected name change from {} to {}'.format(self.repo,
-                                                                         repo_data['full_name']))
+                    log.info('Detected name change from %s to %s', self.repo, repo_data['full_name'])
                     self.set_repo(repo_data['full_name'])
                     # request the feed from the new location
                     return self.get_releases_feed_contents(rename_checked=False)
@@ -501,17 +497,22 @@ class GitHubRepoSession(ProjectHolder):
         feed = feedparser.parse(feed_contents)
         if 'bozo' in feed and feed['bozo'] == 1 and 'bozo_exception' in feed:
             exc = feed.bozo_exception
-            log.info("Failed to parse feed: {}".format(exc.getMessage()))
+            log.info("Failed to parse feed: %s", exc.getMessage())
             return None
         if not feed.entries:
             log.info('Feed has no elements. Means no tags and no releases')
             return []
         return feed.entries
 
+    def enrich_release_info(self, release):
+        """Enrich release info with data from repo."""
+        release['install_name'] = self.name
+        return release
+
     def get_latest(self, pre_ok=False, major=None):
         """
         Get the latest release satisfying "pre-releases are OK" or major/branch constraints
-        Strives to fetch formal API release if it exists, because it has useful information
+        Strive to fetch formal API release if it exists, because it has useful information
         like assets.
         """
         # data of selected tag, always contains ['version', 'tag_name', 'tag_date', 'type'] will
@@ -524,24 +525,44 @@ class GitHubRepoSession(ProjectHolder):
 
         # if major, filter out tags to check for major
 
+        seen_semver = False
+
         feed_entries = self.get_releases_feed_entries()
         if feed_entries:
             for tag in feed_entries:
                 # https://github.com/apache/incubator-pagespeed-ngx/releases/tag/v1.13.35.2-stable
                 tag_name = tag['link'].split('/')[-1]
-                log.info('Checking tag {}'.format(tag_name))
+                tag_name = unquote(tag_name)
+                log.info('Checking tag %s', tag_name)
                 version = self.sanitize_version(tag_name, pre_ok, major)
                 if not version:
-                    log.info('We did not find a valid version in {} tag'.format(tag_name))
+                    log.info('We did not find a valid version in %s tag', tag_name)
                     continue
-                if ret and ret['version'] >= version:
+                if version.is_semver():
+                    seen_semver = True
+                comparable = ret and ret['version'].is_semver() == version.is_semver()
+                if not comparable:
+                    log.info('Tag %s is not comparable to current selection', tag_name)
+                # current tag and chosen tag are only comparable if they are both same semver or not
+                if comparable and ret['version'] > version:
                     log.info(
-                        'Tag {} does not contain newer version than we already found'.format(tag_name)
+                        'Tag %s does not contain newer version than we already found', tag_name
+                    )
+                    continue
+                # if we have seen a semver tag, then any non-semver can be discarded
+                if seen_semver and not version.is_semver():
+                    log.info(
+                        'Tag %s is not a semver and we already found a semver', tag_name
                     )
                     continue
                 tag_date = parser.parse(tag['updated'])
+                if ret and ret['version'] == version and ret['tag_date'] >= tag_date:
+                    log.info(
+                        'Tag %s matches already selected version and is not newer', tag_name
+                    )
+                    continue
                 if ret and tag_date + timedelta(days=365) < ret['tag_date']:
-                    log.info('The version {} is newer, but is too old!'.format(version))
+                    log.info('The version %s is newer, but is too old!', version)
                     break
                 # we always want to return formal release if it exists, because it has useful data
                 # grab formal release via APi to check for pre-release mark
@@ -552,7 +573,7 @@ class GitHubRepoSession(ProjectHolder):
                 else:
                     if self.having_asset:
                         continue
-                    log.info('No formal release for tag {}'.format(tag_name))
+                    log.info('No formal release for tag %s', tag_name)
                     tag['tag_name'] = tag_name
                     tag['tag_date'] = tag_date
                     tag['version'] = version
@@ -562,13 +583,13 @@ class GitHubRepoSession(ProjectHolder):
                     tag.pop('updated_parsed', None)
                     tag.pop('published_parsed', None)
                     ret = tag
-                    log.info("Selected version as current selection: {}.".format(version))
+                    log.info("Selected version as current selection: %s.", version)
 
         # we are good with release from feeds only without looking at the API
         # simply because feeds list stuff in order of recency
         # however, still use /tags unless releases.atom has data within a year
         if ret and ret['tag_date'].replace(tzinfo=None) > (datetime.utcnow() - timedelta(days=365)):
-            return ret
+            return self.enrich_release_info(ret)
 
         log.info('Feed contained none or only tags older than 1 year. Switching to API')
 
@@ -591,7 +612,7 @@ class GitHubRepoSession(ProjectHolder):
             # only formal releases which we enumerated above already, have assets
             # so there is no point looking in the tags/graphql below
             # return whatever we got
-            return ret
+            return self.enrich_release_info(ret)
 
         # formal release may not exist at all, or be "late/old" in case
         # actual release is only a simple tag so let's try /tags
@@ -601,7 +622,7 @@ class GitHubRepoSession(ProjectHolder):
         else:
             ret = self.find_in_tags(ret, pre_ok, major)
 
-        return ret
+        return self.enrich_release_info(ret)
 
     def set_matching_formal_release(self, ret, formal_release, version, pre_ok,
                                     data_type='release'):
@@ -609,7 +630,7 @@ class GitHubRepoSession(ProjectHolder):
         if not pre_ok and formal_release['prerelease']:
             log.info(
                 "Found formal release for this tag which is unwanted "
-                "pre-release: {}.".format(version))
+                "pre-release: %s.", version)
             return ret
         if self.having_asset:
             if 'assets' not in formal_release or not formal_release['assets']:
@@ -633,7 +654,7 @@ class GitHubRepoSession(ProjectHolder):
         formal_release['tag_date'] = parser.parse(formal_release['published_at'])
         formal_release['version'] = version
         formal_release['type'] = data_type
-        log.info("Selected version as current selection: {}.".format(formal_release['version']))
+        log.info("Selected version as current selection: %s.", formal_release['version'])
         return formal_release
 
     def try_get_official(self, repo):
@@ -643,7 +664,7 @@ class GitHubRepoSession(ProjectHolder):
             str: updated repo
         """
         official_repo = "{repo}/{repo}".format(repo=repo)
-        log.info('Checking existence of {}'.format(official_repo))
+        log.info('Checking existence of %s', official_repo)
         r = self.get('https://{}/{}/releases.atom'.format(self.hostname, official_repo))
         # API requests are varied by cookie, we don't want serializer for cache fail because of that
         self.cookies.clear()
