@@ -16,10 +16,6 @@ TOKEN_PRO_TIP = 'ProTip: set GITHUB_API_TOKEN env var as per ' \
                 'https://github.com/dvershinin/lastversion#tips'
 
 
-def set_matching_from_tag(ret, tag):
-    ret = tag
-
-
 def asset_matches(asset, search, regex_matching):
     """Check if the asset equals to string or satisfies a regular expression
     Args:
@@ -67,7 +63,7 @@ class GiteaRepoSession(ProjectHolder):
             cache = {}
         try:
             if repo in cache and time.time() - cache[repo]['updated_at'] < 3600 * 24 * 30:
-                log.info("Found {} in repo short name cache".format(repo))
+                log.info("Found %s in repo short name cache", repo)
                 if not cache[repo]['repo']:
                     raise BadProjectError(
                         'No project found on GitHub for search query: {}'.format(repo)
@@ -75,7 +71,7 @@ class GiteaRepoSession(ProjectHolder):
                 # return cache[repo]['repo']
         except TypeError:
             pass
-        log.info("Making query against GitHub API to search repo {}".format(repo))
+        log.info("Making query against GitHub API to search repo %s", repo)
         r = self.get(
             '{}/search/repositories'.format(self.api_base),
             params={
@@ -101,9 +97,7 @@ class GiteaRepoSession(ProjectHolder):
         }
         try:
             with open(cache_repo_names_file, 'w') as writer:
-                writer.write(
-                    json.dumps(cache)
-                )
+                json.dump(cache, writer)
         except (IOError, ValueError):
             pass
         if not full_name:
@@ -117,9 +111,7 @@ class GiteaRepoSession(ProjectHolder):
         # dict holding repo/owner to feed contents of releases atom
         self.feed_contents = {}
         self.rate_limited_count = 0
-        self.api_token = os.getenv("GITHUB_API_TOKEN")
-        if not self.api_token:
-            self.api_token = os.getenv("GITHUB_TOKEN")
+        self.api_token = os.getenv("GITEA_API_TOKEN")
         self.hostname = hostname
         if not self.hostname:
             self.hostname = self.DEFAULT_HOSTNAME
@@ -138,11 +130,11 @@ class GiteaRepoSession(ProjectHolder):
             official_repo = self.try_get_official(repo)
             if official_repo:
                 repo = official_repo
-                log.info('Using official repo {}'.format(repo))
+                log.info('Using official repo %s', repo)
             else:
                 repo = self.find_repo_by_name_only(repo)
                 if repo:
-                    log.info('Using repo {} obtained from search API'.format(self.repo))
+                    log.info('Using repo %s obtained from search API', self.repo)
                 else:
                     return
         self.set_repo(repo)
@@ -153,7 +145,7 @@ class GiteaRepoSession(ProjectHolder):
     def get(self, url, **kwargs):
         """Send GET request and account for GitHub rate limits and such."""
         r = super(GiteaRepoSession, self).get(url, **kwargs)
-        log.info('Got HTTP status code {} from {}'.format(r.status_code, url))
+        log.info('Got HTTP status code %s from %s', r.status_code, url)
         if r.status_code == 401:
             if self.api_token:
                 raise ApiCredentialsError('API request was denied despite using an API token. '
@@ -291,7 +283,7 @@ class GiteaRepoSession(ProjectHolder):
         if not pre_ok and formal_release['prerelease']:
             log.info(
                 "Found formal release for this tag which is unwanted "
-                "pre-release: {}.".format(version))
+                "pre-release: %s.", version)
             return ret
         if self.having_asset:
             if 'assets' not in formal_release or not formal_release['assets']:
@@ -315,7 +307,7 @@ class GiteaRepoSession(ProjectHolder):
         formal_release['tag_date'] = parser.parse(formal_release['published_at'])
         formal_release['version'] = version
         formal_release['type'] = data_type
-        log.info("Selected version as current selection: {}.".format(formal_release['version']))
+        log.info("Selected version as current selection: %s.", formal_release['version'])
         return formal_release
 
     def try_get_official(self, repo):
@@ -325,7 +317,7 @@ class GiteaRepoSession(ProjectHolder):
             str: updated repo
         """
         official_repo = "{repo}/{repo}".format(repo=repo)
-        log.info('Checking existence of {}'.format(official_repo))
+        log.info('Checking existence of %s', official_repo)
         r = self.get('https://{}/{}/releases.atom'.format(self.hostname, official_repo))
         # API requests are varied by cookie, we don't want serializer for cache fail because of that
         self.cookies.clear()
