@@ -1,10 +1,9 @@
 # -*- coding: utf-8 -*-
-
+import json
 import logging
 import os
 import platform
 import re
-
 import requests
 from appdirs import user_cache_dir
 from cachecontrol import CacheControlAdapter
@@ -19,11 +18,34 @@ from .__about__ import __version__
 # information for a project based on its URL or name (see LocalVersionSession)
 # it is instantiated with a particular project in mind/set, but also has some methods for
 # stuff like searching one
-from .utils import asset_does_not_belong_to_machine
+from .utils import asset_does_not_belong_to_machine, ensure_directory_exists
 
 log = logging.getLogger(__name__)
 
 
+def matches_filter(filter_s, positive, version_s):
+    """Check if version string matches a filter string.
+
+    Args:
+        filter_s (str):  Filter string.
+        positive (bool): Whether filter is positive or negative.
+        version_s (str): Version string, often a tag name.
+
+    Returns:
+        bool: True if version matches filter, False otherwise.
+    """
+    if not filter_s:
+        return True
+
+    if filter_s.startswith('!'):
+        positive = not positive
+        filter_s = filter_s[1:]
+    if filter_s.startswith('~'):
+        filter_s = re.compile(r'{}'.format(filter_s.lstrip('~')))
+        return positive == bool(re.search(filter_s, version_s))
+    return positive == bool(filter_s in version_s)
+
+
 class ProjectHolder(requests.Session):
     """Generic project holder class abstracts a web-accessible project storage."""
 
@@ -57,14 +79,16 @@ class ProjectHolder(requests.Session):
         app_name = __name__.split('.')[0]
 
         self.cache_dir = user_cache_dir(app_name)
-        log.info("Using cache directory: {}.".format(self.cache_dir))
+        log.info("Using cache directory: %s.", self.cache_dir)
         self.cache = FileCache(self.cache_dir)
         cache_adapter = CacheControlAdapter(cache=self.cache)
         self.mount("http://", cache_adapter)
         self.mount("https://", cache_adapter)
 
+        self.names_cache_filename = "{}/repos.json".format(self.cache_dir)
+
         self.headers.update({'User-Agent': '{}/{}'.format(app_name, __version__)})
-        log.info('Created instance of {}'.format(type(self).__name__))
+        log.info('Created instance of %s', type(self).__name__)
         self.branches = None
         self.only = None
         self.exclude = None
@@ -78,6 +102,27 @@ class ProjectHolder(requests.Session):
         self.feed_url = None
         self.even = False
 
+    def get_name_cache(self):
+        """Return name cache from file."""
+        if not os.path.exists(self.names_cache_filename):
+            return {}
+        try:
+            with open(self.names_cache_filename, 'r') as reader:
+                cache = json.load(reader)
+            return cache
+        except (IOError, ValueError) as e:
+            log.warning("Error reading cache file: %s", e)
+            return {}
+
+    def update_name_cache(self, cache_data):
+        """Update name cache file with new data."""
+        try:
+            ensure_directory_exists(self.cache_dir)
+            with open(self.names_cache_filename, 'w') as writer:
+                json.dump(cache_data, writer)
+        except (IOError, ValueError) as e:
+            log.warning("Error writing to cache file: %s", e)
+
     def is_valid(self):
         """Check if project holder is valid instance."""
         return self.feed_url or self.name
@@ -90,14 +135,14 @@ class ProjectHolder(requests.Session):
         """Sets "only" tag selector for this holder."""
         self.only = only
         if only:
-            log.info('Only considering tags with "{}"'.format(only))
+            log.info('Only considering tags with "%s"', only)
         return self
 
     def set_exclude(self, exclude):
         """Sets "exclude" tag selector for this holder."""
         self.exclude = exclude
         if exclude:
-            log.info('Only considering tags without "{}"'.format(exclude))
+            log.info('Only considering tags without "%s"', exclude)
         return self
 
     def set_even(self, even):
@@ -111,7 +156,7 @@ class ProjectHolder(requests.Session):
         """Sets "having_asset" selector for this holder."""
         self.having_asset = having_asset
         if having_asset:
-            log.info('Only considering releases with asset "{}"'.format(having_asset))
+            log.info('Only considering releases with asset "%s"', having_asset)
         return self
 
     @classmethod
@@ -133,11 +178,11 @@ class ProjectHolder(requests.Session):
         if repo.startswith(('https://', 'http://')):
             for url in cls.KNOWN_REPO_URLS:
                 if repo.startswith((url, "https://{}".format(url), "http://{}".format(url))):
-                    log.info('{} Starts with {}'.format(repo, url))
+                    log.info('%s Starts with %s', repo, url)
                     return cls.KNOWN_REPO_URLS[url]
         else:
             if repo.lower() in cls.KNOWN_REPOS_BY_NAME:
-                log.info('Selecting known repo {}'.format(repo))
+                log.info('Selecting known repo %s', repo)
                 return cls.KNOWN_REPOS_BY_NAME[repo.lower()]
         return False
 
@@ -159,62 +204,66 @@ class ProjectHolder(requests.Session):
     def matches_major_filter(self, version, major):
         if self.branches and major in self.branches and \
                 re.search(r"{}".format(self.branches[major]), str(version)):
-            log.info('{} matches major {}'.format(version, self.branches[major]))
+            log.info('%s matches major %s', version, self.branches[major])
             return True
         if str(version).startswith('{}.'.format(major)):
-            log.info('{} is under the desired major {}'.format(
-                version, major))
+            log.info('%s is under the desired major %s', version, major)
             return True
         if str(version) == major:
             return True
         return False
 
-    def matches_only(self, version_s):
-        if not self.only:
-            return True
-        search = self.only
-        positive = True
-        if search.startswith('!'):
-            positive = False
-            search = search[1:]
-        if search.startswith('~'):
-            search = r'{}'.format(self.only.lstrip('~'))
-            return positive == bool(re.search(search, version_s))
-        return positive == bool(search in version_s)
-
     def sanitize_version(self, version_s, pre_ok=False, major=None):
-        """Extract version from tag name."""
-        log.info("Sanitizing string {} as a satisfying version.".format(version_s))
-        res = False
-        if not self.matches_only(version_s):
-            log.info('"{}" does not match the "only" constraint "{}"'.format(version_s, self.only))
-            return res
+        """
+        Extract a version from tag name; that satisfies this holder's filters, etc.
+
+        Returns:
+            Version or None: The return value can be a Version object or None.
+        """
+        log.info("Sanitizing string %s as a satisfying version.", version_s)
+
+        # for libssh2-x.x.x should remove project name prefix to prevent `2` going into the version
+        prefix = "{}-".format(self.name)
+        if version_s.startswith(prefix):
+            version_s = version_s[len(prefix):]
+            log.info("Removed project name prefix, working now on string '%s'", version_s)
+
+        res = None
+
+        if not matches_filter(self.only, True, version_s):
+            log.info('"%s" does not match the "only" constraint "%s"', version_s, self.only)
+            return None
+
+        if not matches_filter(self.exclude, False, version_s):
+            log.info('"%s" does not match the "exclude" constraint "%s"', version_s, self.exclude)
+            return None
+
         try:
             char_fix_required = self.repo in self.LAST_CHAR_FIX_REQUIRED_ON
             v = Version(version_s, char_fix_required=char_fix_required)
             if not v.is_prerelease or pre_ok:
-                log.info("Parsed as Version OK. String representation: {}.".format(v))
+                log.info("Parsed as Version OK. String representation: %s.", v)
                 res = v
             else:
-                log.info("Parsed as unwanted pre-release version: {}.".format(v))
+                log.info("Parsed as unwanted pre-release version: %s.", v)
         except InvalidVersion:
-            log.info("Failed to parse {} as Version.".format(version_s))
+            log.info("Failed to parse %s as Version.", version_s)
             # attempt to remove extraneous chars and revalidate
             # we use findall for cases where "tag" may be 'foo/2.x/2.45'
             matches = re.findall(r'([0-9]+([.][0-9x]+)+(rc[0-9]?)?)', version_s)
             for s in matches:
                 version_s = s[0]
-                log.info("Sanitized tag name value to {}.".format(version_s))
-                # 1.10.x is a dev release without clear version, so even pre ok will not get it
+                log.info("Sanitized tag name value to %s.", version_s)
+                # 1.10.x is a dev release without a clear version, so even pre ok will not get it
                 if not version_s.endswith('.x'):
-                    # we know regex is valid version format, so no need to try catch
+                    # we know regex is a valid version format, so no need to try catch
                     res = Version(version_s)
                 if res:
-                    # satisfy on the first matched version-like string, e.g. 5.2.6-3.12
+                    # Satisfy on the first matched version-like string, e.g. 5.2.6-3.12
                     break
             if not matches:
                 log.info("Did not find anything that looks like a version in the tag")
-                # as a last resort, let's try to convert underscores to dots, while stripping out
+                # As the last resort, let's try to convert underscores to dots, while stripping out
                 # any "alphanumeric_". many hg repos do this, e.g. PROJECT_1_2_3
                 parts = version_s.split('_')
                 if len(parts) >= 2 and parts[0].isalpha():
@@ -224,19 +273,20 @@ class ProjectHolder(requests.Session):
                         v = Version(version_s)
                         if not v.is_prerelease or pre_ok:
                             log.info("Parsed as Version OK")
-                            log.info("String representation of version is {}.".format(v))
+                            log.info("String representation of version is %s.", v)
                             res = v
                         else:
-                            log.info("Parsed as unwanted pre-release version: {}.".format(v))
+                            log.info("Parsed as unwanted pre-release version: %s.", v)
                     except InvalidVersion:
                         log.info('Still not a valid version after applying underscores fix')
         # apply --major filter
         if res and major and not self.matches_major_filter(res, major):
-            log.info('{} is not under the desired major {}'.format(
-                version_s, major))
-            res = False
+            log.info('%s is not under the desired major %s', version_s, major)
+            return None
+
         if res and self.even and not res.even:
-            return False
+            return None
+
         return res
 
     def _type(self):
@@ -264,9 +314,7 @@ class ProjectHolder(requests.Session):
 
     def get_assets(self, release, short_urls, assets_filter=None):
         urls = []
-        assets = []
-        if 'assets' in release:
-            assets = release['assets']
+        assets = release.get('assets', [])
         arch_matched_assets = []
         if not assets_filter and platform.machine() in ['x86_64', 'AMD64']:
             for asset in assets:
