@@ -1,13 +1,14 @@
+"""Version class for lastversion"""
 import re
-
+from datetime import datetime
 from packaging.version import Version as PackagingVersion, InvalidVersion
 
 
 class Version(PackagingVersion):
     """
     This class abstracts handling of a project's versions. It implements the
-    scheme defined in PEP 440. A `Version` instance is comparison
-    aware and can be compared and sorted using the standard Python interfaces.
+    scheme defined in PEP 440. A `Version` instance is comparison-aware and
+    can be compared and sorted using the standard Python interfaces.
 
     This class is descendant from Version found in `packaging.version`,
     and implements some additional, "AI"-like normalization during instantiation.
@@ -24,6 +25,74 @@ class Version(PackagingVersion):
         self.fixed_letter_post_release = True
         return match.group(1) + '.post' + str(ord(match.group(2)))
 
+    def is_semver(self):
+        """Check if this a semantic version"""
+        return self.base_version.count('.') == 2
+
+    @staticmethod
+    def part_to_pypi(part):
+        """
+        Convert a version part to a PyPI compatible string
+        See https://peps.python.org/pep-0440/
+        Helps devel releases to be correctly identified
+        See https://www.python.org/dev/peps/pep-0440/#developmental-releases
+        """
+        if part in ['devel', 'test', 'dev']:
+            part = 'dev0'
+        elif part in ['alpha']:
+            # "4.3.0-alpha"
+            part = 'a0'
+        elif part in ['beta']:
+            # "4.3.0-beta"
+            part = 'b0'
+            # if part starts with rc<num>., discard non-relevant info while preserving RC level
+        elif re.search('^rc(\\d+)\\.', part):
+            # rc2.windows.1 => rc2.post1
+            sub_parts = part.split('.')
+            part = sub_parts[0]
+            for sub in sub_parts[1:]:
+                if sub.isdigit():
+                    # use first numeric as post-release to RC
+                    part += ".post" + sub
+        else:
+            # help post (patch) releases to be correctly identified (e.g. Magento 2.3.4-p2)
+            # p12 => post12
+            part = re.sub('^p(\\d+)$', 'post\\1', part, 1)
+        if part.isalpha():
+            # it's meaningless to us if it has only letters
+            part = None
+        return part
+
+    @staticmethod
+    def join_dashed_number_status(version):
+        """
+        Join status with its number when separated by dash in a version string.
+        E.g., 4.27-chaos-preview-3 -> 4.27-chaos-pre3
+        Helps devel releases to be correctly identified
+        # https://www.python.org/dev/peps/pep-0440/#developmental-releases
+
+        Args:
+            version:
+
+        Returns:
+            str:
+        """
+        version = re.sub('-devel$', '-dev0', version, 1)
+        # help post (patch) releases to be correctly identified (e.g., Magento 2.3.4-p2)
+        version = re.sub('-p(\\d+)$', '-post\\1', version, 1)
+
+        # 4.27-chaos-preview-3 -> 4.27-chaos-pre3
+        version = re.sub('-preview-(\\d+)', '-pre\\1', version, 1)
+        # 5.0.0-early-access-2 -> 5.0.0-alpha2
+        version = re.sub('-early-access-(\\d+)', '-alpha\\1', version, 1)
+        # v4.0.0-pre-0 -> v4.0.0-pre0
+        version = re.sub('-pre-(\\d+)', '-pre\\1', version, 1)
+
+        # v0.16.0-beta.rc4 -> v0.16.0-beta4
+        # both beta and rc? :) -> beta
+        version = re.sub(r'-beta[-.]rc(\d+)', '-beta\\1', version, 1)
+        return version
+
     def __init__(self, version, char_fix_required=False):
         """Instantiate the `Version` object.
 
@@ -33,33 +102,18 @@ class Version(PackagingVersion):
         """
         self.fixed_letter_post_release = False
 
-        # 4.27-chaos-preview-3 -> 4.27-chaos-pre3
-        version = re.sub('-preview-(\\d+)', '-pre\\1', version, 1)
-        # 5.0.0-early-access-2 -> 5.0.0-alpha2
-        version = re.sub('-early-access-(\\d+)', '-alpha\\1', version, 1)
+        # Join status with its number, e.g., preview-3 -> pre3
+        version = self.join_dashed_number_status(version)
 
-        # many times they would tag foo-1.2.3 which would parse to LegacyVersion
-        # we can avoid this, by reassigning to what comes after the dash:
+        # parse out version components separated by dash
         parts = version.split('-')
 
-        # TODO test v5.12-rc1-dontuse -> v5.12.rc1
-        # go through parts separated by dot, detect beta level, and weed out numberless info:
+        # go through parts which were separated by dash, normalize and exclude irrelevant
         parts_n = []
         for part in parts:
-            # help devel releases to be correctly identified
-            # https://www.python.org/dev/peps/pep-0440/#developmental-releases
-            if part in ['devel', 'test', 'dev']:
-                part = 'dev0'
-            elif part in ['beta']:
-                # "4.3.0-beta"
-                part = 'b0'
-            else:
-                # help post (patch) releases to be correctly identified (e.g. Magento 2.3.4-p2)
-                # p12 => post12
-                part = re.sub('^p(\\d+)$', 'post\\1', part, 1)
-            if not part.isalpha():
+            part = self.part_to_pypi(part)
+            if part:
                 parts_n.append(part)
-
         if not parts_n:
             raise InvalidVersion("Invalid version: '{0}'".format(version))
         # remove *any* non-digits which appear at the beginning of the version string
@@ -74,7 +128,7 @@ class Version(PackagingVersion):
         if char_fix_required:
             version = re.sub('(\\d)([a-z])$', self.fix_letter_post_release, version, 1)
         # release-3_0_2 is often seen on Mercurial holders
-        # note that above code removes "release-" already, so we are left with "3_0_2"
+        # note that the above code removes "release-" already, so we are left with "3_0_2"
         if re.search(r'^(?:\d+_)+(?:\d+)', version):
             version = version.replace('_', '.')
         # finally, split by dot "delimiter", see if there are common words which are definitely
@@ -142,15 +196,33 @@ class Version(PackagingVersion):
         # type: () -> int
         return self.release[2] if len(self.release) >= 3 else 0
 
+    @staticmethod
+    def is_not_date(num):
+        """Helper function to determine if a number is not a date"""
+        num_str = str(num)
+        try:
+            # Attempt to parse the number as a date
+            datetime.strptime(num_str, '%Y%m%d')
+            return False
+        except ValueError:
+            # If parsing fails, the number is not a date
+            return True
+
     @property
     def is_prerelease(self):
+        """
+        Version is a prerelease if it contains all the following:
+        * 90+ micro component
+        * no date in micro component
+        """
         # type: () -> bool
-        if self.major and self.minor and self.micro >= 90:
+        if self.major and self.minor and self.micro >= 90 and self.is_not_date(self.micro):
             return True
         return self.dev is not None or self.pre is not None
 
     @property
     def even(self):
+        """Check if this is an even minor version"""
         return self.minor and not self.minor % 2
 
     def sem_extract_base(self, level=None):
@@ -162,9 +234,9 @@ class Version(PackagingVersion):
             # get major
             return Version(str(self.major))
         if level == 'minor':
-            return Version("{}.{}".format(self.major, self.minor))
+            return Version(f"{self.major}.{self.minor}")
         if level == 'patch':
-            return Version("{}.{}.{}".format(self.major, self.minor, self.micro))
+            return Version(f"{self.major}.{self.minor}.{self.micro}")
         return self
 
     def __str__(self):
