@@ -18,6 +18,12 @@ import re
 import sys
 from os.path import expanduser
 
+try:
+    # noinspection PyCompatibility
+    from pathlib import Path
+except ImportError:
+    # noinspection PyUnresolvedReferences
+    from pathlib2 import Path  # python 2 backport
 import yaml
 from packaging.version import InvalidVersion
 from six.moves.urllib.parse import urlparse
@@ -30,7 +36,7 @@ from .__about__ import __self__
 from .argparse_version import VersionAction
 from .spdx_id_to_rpmspec import rpmspec_licenses
 from .utils import download_file, extract_file, rpm_installed_version, ApiCredentialsError, \
-    BadProjectError
+    BadProjectError, extract_appimage_desktop_file
 
 log = logging.getLogger(__name__)
 
@@ -38,20 +44,20 @@ log = logging.getLogger(__name__)
 def latest(repo, output_format='version', pre_ok=False, assets_filter=None,
            short_urls=False, major=None, only=None, at=None,
            having_asset=None, exclude=None, even=False):
-    """Find the latest release version for a project.
+    r"""Find the latest release version for a project.
 
     Args:
         major (str): Only consider versions which are "descendants" of this major version string
         short_urls (bool): Whether we should try to return shorter URLs for release data
-        assets_filter (str): Regular expression for filtering assets for the latest release
+        assets_filter (Union[str, Pattern]): Regular expression for filtering assets for the latest release
         only (str): Only consider tags with this text. Useful for repos with multiple projects.
                     The argument supports negation and regular expressions. To indicate a regex,
                     start it with tilde sign, to negate the expression, start it with exclamation
                     point. See ``Examples``.
         repo (str): Repository specifier in any form.
-        output_format (str): Affects return format. Possible values `version`, `json`, `dict`,
+        output_format (str): Affects the return format. Possible values `version`, `json`, `dict`,
                              `assets`, `source`, `tag`.
-        pre_ok (bool): Specifies whether pre-releases can be accepted as newer version.
+        pre_ok (bool): Specifies whether pre-releases can be accepted as a newer version.
         at (str): Specifies repo hosting more precisely, only useful if repo argument was
                   specified as one word.
         having_asset (Union[str, bool]): Only consider releases with the given asset.
@@ -71,7 +77,7 @@ def latest(repo, output_format='version', pre_ok=False, assets_filter=None,
         <Version('3.0.7')>
 
     Returns:
-        Version: Newer version object, if found and `output_format` is `version`.
+        Union[Version, dict]: Newer version object, if found and `output_format` is `version`.
     Returns:
         str: Single string containing tag, if found and `output_format` is `tag`
 
@@ -136,8 +142,7 @@ def latest(repo, output_format='version', pre_ok=False, assets_filter=None,
                 if current_version != 'x':
                     repo_data['current_version'] = Version(current_version)
             except InvalidVersion:
-                log.critical('Failed to parse current version in {}. Tried {}'.format(
-                    repo, current_version))
+                log.critical('Failed to parse current version in %s. Tried %s', repo, current_version)
                 sys.exit(1)
             if upstream_name:
                 repo_data['name'] = upstream_name
@@ -147,10 +152,10 @@ def latest(repo, output_format='version', pre_ok=False, assets_filter=None,
                 repo_data['spec_name'] = '%{name}'
             if upstream_github:
                 repo = "{}/{}".format(upstream_github, repo_data['name'])
-                log.info('Discovered GitHub repo {} from .spec file'.format(repo))
+                log.info('Discovered GitHub repo %s from .spec file', repo)
             elif spec_repo:
                 repo = spec_repo
-                log.info('Discovered explicit repo {} from .spec file'.format(repo))
+                log.info('Discovered explicit repo %s from .spec file', repo)
             elif spec_url:
                 repo = spec_url
 
@@ -258,31 +263,17 @@ def check_version(value):
     In lastversion CLI app, this is used as argument parser helper for --newer-than (-gt) option.
 
     Args:
-        value (str): Free-format string which is meant to contain user-supplied version
+        value (str): Free-format string which is meant to contain a user-supplied version
 
     Raises:
-        argparse.ArgumentTypeError: Exception in case version was not found in the input string
+        argparse.ArgumentTypeError: Exception in a case version was not found in the input string
 
     Returns:
         Version: Parsed version object
 
     """
-    """
-    Argument parser helper for --newer-than (-gt) option
-    :param value:
-    :type value:
-    :return:
-    :rtype:
-    """
-    try:
-        # TODO use sanitize_version so that we can just pass tags as values
-        # help devel releases to be correctly identified
-        # https://www.python.org/dev/peps/pep-0440/#developmental-releases
-        value = re.sub('-devel$', '.dev0', value, 1)
-        # help post (patch) releases to be correctly identified (e.g. Magento 2.3.4-p2)
-        value = re.sub('-p(\\d+)$', '.post\\1', value, 1)
-        value = Version(value)
-    except InvalidVersion:
+    value = parse_version(value)
+    if not value:
         raise argparse.ArgumentTypeError("%s is an invalid version value" % value)
     return value
 
@@ -308,7 +299,7 @@ def get_rpm_packager():
 def update_spec(repo, res, sem='minor'):
     print(res['version'])
     if 'current_version' not in res or res['current_version'] < res['version']:
-        log.info('Updating spec {} with semantic {}'.format(repo, sem))
+        log.info('Updating spec %s with semantic %s', repo, sem)
         if 'current_version' in res and len(res['version'].release) >= 3:
             current_major = res['current_version'].release[0]
             latest_major = res['version'].release[0]
@@ -364,9 +355,32 @@ def update_spec(repo, res, sem='minor'):
         f.write("\n".join(out))
 
 
-def main():
-    """The entrypoint to CLI app."""
+def install_app_image(url, install_name):
+    """Install an AppImage from a URL to `~/Applications/<install_name>`
+
+    Args:
+        url (str): URL where AppImage file is hosted
+        install_name (str): Short name that the AppImage will be renamed to
+    """
+    home_dir = os.path.expanduser('~')
+    apps_dir = os.path.join(home_dir, 'Applications')
+    app_file_name = os.path.join(apps_dir, f"{install_name}.AppImage")
+
+    Path(apps_dir).mkdir(exist_ok=True, parents=True)
+    download_file(url, app_file_name)
+    os.chmod(app_file_name, 0o755)  # skipcq: BAN-B103
+    extract_appimage_desktop_file(app_file_name)
+
+
+def main(argv=None):
+    """
+    The entrypoint to CLI app.
+
+    Args:
+        argv: List of arguments, helps test CLI without resorting to subprocess module.
+    """
     epilog = None
+
     if "GITHUB_API_TOKEN" not in os.environ and "GITHUB_TOKEN" not in os.environ:
         epilog = TOKEN_PRO_TIP
     parser = argparse.ArgumentParser(description='Find the latest software release.',
@@ -405,7 +419,7 @@ def main():
     parser.add_argument('-b', '--major', '--branch', metavar='MAJOR',
                         help="Only consider releases of a specific major "
                              "version, e.g. 2.1.x")
-    parser.add_argument('--only', metavar='REGEX', 
+    parser.add_argument('--only', metavar='REGEX',
                         help="Only consider releases containing this text. "
                              "Useful for repos with multiple projects inside")
     parser.add_argument('--exclude', metavar='REGEX',
@@ -431,7 +445,7 @@ def main():
                         pre=False, assets=False, newer_than=False, filter=False,
                         shorter_urls=False, major=None, assumeyes=False, at=None,
                         having_asset=None, even=False)
-    args = parser.parse_args()
+    args = parser.parse_args(argv)
 
     if args.repo == "self":
         args.repo = __self__
@@ -459,7 +473,7 @@ def main():
 
     if args.verbose:
         logger.setLevel(logging.DEBUG)
-        log.info("Verbose {} level output.".format(args.verbose))
+        log.info("Verbose %s level output.", args.verbose)
         if args.verbose >= 2:
             cachecontrol_logger = logging.getLogger('cachecontrol')
             cachecontrol_logger.removeHandler(logging.NullHandler())
@@ -488,16 +502,16 @@ def main():
                 print("Stable: {}".format(not v.is_prerelease))
             else:
                 print(v)
-            sys.exit(0)
+            return sys.exit(0)
 
     if args.action == 'install':
         # we can only install assets
         args.format = 'json'
         if args.having_asset is None:
-            args.having_asset = r'~\.rpm$'
+            args.having_asset = r'~\.(AppImage|rpm)$'
             try:
                 import apt
-                args.having_asset = r'~\.deb$'
+                args.having_asset = r'~\.(AppImage|deb)$'
             except ImportError:
                 pass
 
@@ -528,8 +542,8 @@ def main():
         base_compare = parse_version(args.repo)
         if base_compare:
             print(max([args.newer_than, base_compare]))
-            sys.exit(2 if base_compare <= args.newer_than else 0)
-     
+            return sys.exit(2 if base_compare <= args.newer_than else 0)
+
     # other action are either getting release or doing something with release (extend get action)
     try:
         res = latest(args.repo, args.format, args.pre, args.filter,
@@ -555,7 +569,7 @@ def main():
             if len(res) == 1:
                 download_name = args.download
             for url in res:
-                log.info("Downloading {} ...".format(url))
+                log.info("Downloading %s ...", url)
                 download_file(url, download_name)
             sys.exit(0)
 
@@ -565,11 +579,16 @@ def main():
                 # there is only one source, but we need an array
                 res = [res]
             for url in res:
-                log.info("Extracting {} ...".format(url))
+                log.info("Extracting %s ...", url)
                 extract_file(url)
             sys.exit(0)
 
         if args.action == 'install':
+            app_images = [asset for asset in res['assets'] if asset.endswith('.AppImage')]
+            if app_images:
+                return install_app_image(
+                    app_images[0], install_name=res.get('install_name', args.repo)
+                )
             rpms = [asset for asset in res['assets'] if asset.endswith('.rpm')]
             if not rpms:
                 log.error('No assets found to install')
